package com.jiayue.center.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.jiayue.center.entity.SysDept;
import com.jiayue.center.entity.SysMenu;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 *  部门管理Mapper
 *
 * @author xsl
 * @since 2023-03-17
 */
@Mapper
public interface SysDeptMapper extends BaseMapper<SysDept> {
    /**
     * 查询系统部门列表
     *
     * @param sysDept 部门信息
     * @return 部门列表
     */
    public List<SysDept> selectDeptList(SysDept sysDept);


    /**
     * 根据用户ID查询菜单
     *
     * @return 菜单列表
     */
    public List<SysDept> selectDeptTreeAll();

    /**
     * 根据菜单ID查询信息
     *
     * @param deptId 菜单ID
     * @return 菜单信息
     */
    public SysDept selectDeptById(Long deptId);

    /**
     * 是否存在菜单子节点
     *
     * @param deptId 菜单ID
     * @return 结果
     */
    public int hasChildByDeptId(Long deptId);

    /**
     * 新增部门信息
     *
     * @param sysDept 部门信息
     * @return 结果
     */
    public int insertDept(SysDept sysDept);

    /**
     * 修改部门信息
     *
     * @param sysDept 部门信息
     * @return 结果
     */
    public int updateDept(SysDept sysDept);

    /**
     * 删除部门管理信息
     *
     * @param updateBy 修改人
     * @param deptId 菜单ID
     * @return 结果
     */
    public int deleteDeptById(String updateBy,Long deptId);

    /**
     * 校验菜单名称是否唯一
     *
     * @param menuName 菜单名称
     * @param parentId 父菜单ID
     * @return 结果
     */
    public SysDept checkDeptNameUnique(@Param("deptName") String deptName, @Param("parentId") Long parentId);
}
